## Chapter 6 

## Sec 6.1: Basic Ideas: a Book Weight Example 
## Footnote code 
## Plot of weight vs volume: data frame allbacks (DAAG) 
plot(weight ~ volume, data=allbacks, pch=c(16,1)[unclass(cover)]) 
 # unclass(cover) gives the integer codes that identify levels 
## Use with() to specify data frame [text() does not take a 'data' parameter] 
with(allbacks, text(weight ~ volume, labels=paste(1:15),  
                    pos=c(2,4)[unclass(cover)])) 
allbacks.lm <- lm(weight ~ volume+area, data=allbacks) 
summary(allbacks.lm) 
 # for coefficient estimates and SEs, without other output, enter 
 # summary(allbacks.lm)$coef 
## Footnote code 
qt(0.975, 12) 
anova(allbacks.lm) 
## Footnote code 
with(allbacks, cor(volume,area)) 
model.matrix(allbacks.lm) 

## ss 6.1.1: Omission of the intercept term 
allbacks.lm0 <- lm(weight ~ -1+volume+area, data=allbacks) 
summary(allbacks.lm0) 
## Display correlations between estimates of model coefficients 
summary(allbacks.lm, corr=TRUE) 

## ss 6.1.2: Diagnostic plots 
par(mfrow=c(2,2))    # Get all 4 plots on one page 
plot(allbacks.lm0, which=1:4)  
par(mfrow=c(1,1)) 
allbacks.lm13 <- lm(weight ~ -1+volume+area, data=allbacks[-13, ])  
summary(allbacks.lm13) 

## Sec 6.2: The Interpretation of Model Coefficients 

## ss 6.2.1: Times for Northern Irish hill races 
## Footnote code 
## Scatterplot matrix: data frame hills2000 (DAAG) 
## Panel A: untransformed data 
library(lattice) 
splom(~ nihills[, c("dist","climb","time")], cex.labels=1.2, 
      varnames=c("dist\n(miles)","climb\n(feet)", "time\n(hours)")) 
## Panel B: log transformed data 
splom(~ log(nihills[, c("dist","climb","time")]), cex.labels=1.2, 
      varnames=c("dist\n(log miles)", "climb\n(log feet)",  
                 "time\n(log hours)")) 
> nihills.lm <- lm(log(time) ~ log(dist) + log(climb), data = nihills) 
> summary(nihills.lm) 
. . . . 
Coefficients: 
            Estimate Std. Error t value Pr(>|t|) 
(Intercept)  -4.9611     0.2739   -18.1  7.1e-14 
log(dist)     0.6814     0.0552    12.3  8.2e-11 
log(climb)    0.4658     0.0453    10.3  2.0e-09 
 
Residual standard error: 0.0766 on 20 degrees of freedom 
Multiple R-squared: 0.983,	Adjusted R-squared: 0.981  
F-statistic:  583 on 2 and 20 DF,  p-value: <2e-16  
plot(nihills.lm, which=1:4) 

## ss 6.2.2: Plots that show the contribution of individual terms 
yterms <- predict(nihills.lm, type="terms")  
lognihills <- log(nihills) 
names(lognihills) <- paste("log", names(nihills), sep="") 
nihills.lm <- lm(logtime ~ logdist + logclimb, data = lognihills) 
## To show points, specify partial.resid=TRUE 
## For a smooth curve, specify smooth=panel.smooth 
termplot(nihills.lm, partial.resid=TRUE, smooth=panel.smooth,  
         col.res="gray30") 
par(mfrow=c(1,1)) 
## Footnote code 
## Alternatively, plots of partial residuals can be obtained directly 
## from information in nihills.lm 
res <- resid(nihills.lm) 
yhat <- predict(nihills.lm, type="terms") 
plot(lognihills$logdist, yhat[, "loghist"] + res, type="n") 
panel.smooth(lognihills$logdist, yhat[, "logdist"] + res) 
plot(lognihills$logclimb, yhat[, "logclimb"] + res, type="n") 
panel.smooth(lognihills$logclimb, yhat[, "logclimb"] + res) 
lognihills$logGrad <- with(nihills, log(climb/dist)) 
nihillsG.lm <- lm(logtime ~ logdist + logGrad, data=lognihills) 
summary(nihillsG.lm)$coef 
## Footnote code 
with(lognihills, cor(logdist, logGrad)) 
with(lognihills, cor(logdist, logclimb)) 

## ss 6.2.3: Mouse brain weight example 
## Footnote code 
## Scatterplot matrix for data frame litters (DAAG); labels as in figure 
library(lattice) 
splom(~litters, varnames=c("lsize\n\n(litter size)", "bodywt\n\n(Body Weight)", 
      "brainwt\n\n(Brain Weight)")) 
pairs(litters)       # For improved labeling, see the footnote. 
summary(lm(brainwt ~ lsize + bodywt, data = litters))$coef  

## ss 6.2.4: Book dimensions, density and book weight 
## Footnote code 
pairs(log(oddbooks), labels=c("thick\n\nlog(mm)", "breadth\n\nlog(cm)", 
      "height\n\nlog(cm)", "weight\n\nlog(g)")) 
## Footnote code 
volume <- apply(oddbooks[, 1:3], 1, prod)              
area <- apply(oddbooks[, 2:3], 1, prod)                
lob1.lm <- lm(log(weight) ~ log(volume), data=oddbooks)      
lob2.lm <- lm(log(weight) ~ log(thick)+log(area), data=oddbooks) 
lob3.lm <- lm(log(weight) ~ log(thick)+log(breadth)+log(height), 
              data=oddbooks) 
coef(summary(lob1.lm)) 
## Similarly for coefficients and SEs for other models 
coef(summary(lm(log(weight) ~ log(thick), data=oddbooks))) 
book.density <- oddbooks$weight/volume 
bookDensity.lm <- lm(log(book.density) ~ log(area), data=oddbooks) 
coef(summary(bookDensity.lm)) 

## Sec 6.3: Multiple Regression Assumptions, Diagnostics and Efficacy Measures 

## ss 6.3.1: Influential outliers, leverage and Cook's distance 
##                 $^{*}$ Leverage and the hat matrix 
as.vector(hatvalues(nihills.lm))  # as.vector() strips off names 

##                          Cook's distance 
plot(mice12.lm, which=5, add.smooth=FALSE) 
## The points can be plotted using 
## plot(hatvalues(model.matrix(nihills.lm)), residuals(nihills.lm)) 

##              Influence on the regression coefficients 
##               \!\!$^{*}$Additional diagnostic plots 
library(car) 
leverage.plots(allbacks.lm, term.name="volume",  
               identify.points=FALSE) 

## ss 6.3.2: Assessment and Comparison of Regression Models 
##                      $R^2$ and adjusted $R^2$ 
##                     AIC and related statistics 
##           Calculation of\, {\rm R}'s version of the AIC 
## ss 6.3.3: How accurately does the equation predict? 
lognihills <- log(nihills) 
names(lognihills) <- paste("log", names(nihills), sep="") 
nihills.lm <- lm(logtime ~ logdist + logclimb, data = lognihills) 
exp(predict(nihills.lm, interval="confidence"))[1:5, ] 
## Footnote code 
## For the quadratic model 
nihills2.lm <- lm(logtime ~ poly(logdist,2) + logclimb, data = lognihills) 
citimes2 <- exp(predict(nihills2.lm, interval="confidence")) 

## Sec 6.4: A Strategy for Fitting Multiple Regression Models 

## ss 6.4.1: Preliminaries 
##    *Explanatory variables that exhibit a nonlinear relationship 
## ss 6.4.2: Model fitting 
## ss 6.4.3: An example -- Scottish hill race data 
## Footnote code 
## Scatterplot matrix: data frame hills2000 (DAAG), log scales 
splom(~ log(hills2000[, c("dist","climb","time")]), cex.labels=1.2, 
      varnames=c("dist\n(log miles)", "climb\n(log feet)", "time\n(log hours)")) 
## Panel A 
lhills2k.lm <- lm(log(time) ~ log(climb) + log(dist), data = dframe) 
plot(lhills2k.lm, caption="", which=1) 
## Panel B 
lhills2k.lqs <- lqs(log(time) ~ log(climb) + log(dist), data = dframe) 
reres <- residuals(lhills2k.lqs) 
refit <- fitted(lhills2k.lqs) 
big3 <- which(abs(reres) >= sort(abs(reres), decreasing=TRUE)[3]) 
plot(reres ~ refit, xlab="Fitted values (resistant fit)", 
     ylab="Residuals (resistant fit)") 
lines(lowess(reres ~ refit, col=2) 
text(reres[big3] ~ refit[big3], labels=rownames(dframe)[big3], 
     pos=4-2*(refit[big3] > mean(refit)), cex=0.8) 
plot(lhills2k.lm, which=2)  
## Footnote code 
## Probability of a value <= -3 or >= 3  
2 * pnorm(-3)  

##               The contribution of the separate terms 
## Create data frame that has as its columns logs of time, dist & climb 
lhills2k <- log(hills2000[, c("dist", "climb", "time")]) 
names(lhills2k) <- paste("log", names(lhills2k), sep="") 
lhills2k.lm <- lm(logtime ~ logdist+logclimb, data=lhills2k) 
termplot(lhills2k.lm, partial.resid=TRUE, smooth=panel.smooth,  
         col.res="gray30") 

##   A resistant fit that has a polynomial term in \texttt{logdist} 
reres2 <- residuals(lqs(logtime ~ poly(logdist,2)+logclimb, data=lhills2k)) 
reres2[order(abs(reres2), decreasing=TRUE)[1:4]] 

##                         Refining the model 
add1(lhills2k.lm, ~ logdist+I(logdist^2)+logclimb+logdist:logclimb,  
lhills2k.lm2 <- lm(logtime ~ poly(logdist,2)+logclimb, data=lhills2k[-42, ]) 
plot(lhills2k.lm2) 
## Footnote code 
add1(lhills2k.lm2, ~ poly(logdist,2)+logclimb+logdist:logclimb) 

##               The model without the interaction term 
summary(lhills2k.lm)$coef 

##                    Are "errors in x" an issue? 
##                What happens if we do not transform? 

## Sec 6.5: Problems with Many Explanatory Variables 
## ss 6.5.1: Variable selection issues 
##        Variable selection -- a simulation with random data 
## Footnote code 
y <- rnorm(100) 
xx <- matrix(rnorm(4000), ncol = 40) 
dimnames(xx)<- list(NULL, paste("X",1:40, sep="")) 
## Footnote code 
library(leaps) 
xx.subsets <- regsubsets(xx, y, method = "exhaustive", nvmax = 3, nbest = 1) 
subvar <- summary(xx.subsets)$which[3,-1] 
best3.lm <- lm(y ~ -1+xx[, subvar]) 
print(summary(best3.lm, corr = FALSE)) 
rm(xx, y, xx.subsets, subvar, best3.lm) 
  # For once, we note objects that should be removed.  
## Note also our function bestset.noise(). The following 
## call is effectively equivalent to the above. 
bestset.noise(m=100, n=40) 

## Sec 6.6: Multicollinearity 

## ss 6.6.1: A contrived example 
## Footnote code 
## Scatterplot matrix for data frame carprice (DAAG) 
pairs(carprice[, -c(1,8,9)]) 
## Alternative, using the lattice function splom() 
splom(~carprice[, -c(1,8,9)]) 
carprice1.lm <- lm(gpm100 ~ Type+Min.Price+Price+Max.Price+ 
                   Range.Price, data=carprice) 
summary(carprice1.lm)$coef 
alias(carprice1.lm) 
carprice2.lm <- lm(gpm100 ~ Type + Min.Price + Price +  
                   Max.Price + RoughRange, data=carprice) 
summary(carprice2.lm) 

##                An analysis that makes modest sense 
carprice.lm <- lm(gpm100 ~ Type + Price, data = carprice) 
summary(carprice.lm)$coef 
## Footnote code 
summary(carprice.lm)$sigma    # residual standard error when only price is used 
summary(carprice1.lm)$sigma   # residual se when fitting all 3 price variables 

## ss 6.6.2: The variance inflation factor (VIF) 
library (DAAG)     # Be sure to use vif() from the DAAG package 
vif(lm(gpm100 ~ Price, data=carprice))  # Baseline  
vif(carprice1.lm)  # has Min.Price, Price & Max.Price 
vif(carprice2.lm)  # Add RoughRange 
vif(carprice.lm)   # Price is the only price variable 

## ss 6.6.3: Remedies for  multicollinearity 

## Sec 6.7: Multiple Regression Models -- Additional Points 
## ss 6.7.1: Errors in $x$ 
##                   Measurement of dietary intake 
##          A simulation of the effect of measurement error 
##             Errors in variables -- multiple regression 
##        One covariate measured without error -- a simulation 
## ss 6.7.2: Implications for variable selection 
## ss 6.7.3: Confusion between explanatory and response variables 
## Footnote code 
## For convenience, here again are the R commands: 
hills.loglm <- lm(log(dist) ~ log(time)+log(climb), data=hills[-18, ]) 
summary(hills.loglm) 

##                      Unintended correlations 
## ss 6.7.4: Missing explanatory variables 
## ss 6.7.5: \!\!$^{*}$ The use of transformations 
## ss 6.7.6: \!\!$^{*}$ Non-linear methods -- an alternative to transformation? 
hills$climb.mi <- hills$climb/5280 
hills.nls0 <- nls(time ~ (dist^alpha)*(climb.mi^beta), start = 
                  c(alpha = .909, beta = .260), data = hills[-18,]) 
summary(hills.nls0) 
plot(residuals(hills.nls0) ~ predict(hills.nls0)) # residual plot 
hills.nls <- nls(time ~ alpha + beta*dist +  
     gamma*(climb.mi^delta), start=c(alpha = 1, beta = 1,  
     gamma = 1, delta = 1), data=hills[-18, ]) 
summary(hills.nls) 
plot(residuals(hills.nls) ~ predict(hills.nls))   
  # residual plot 

## Sec 6.8: Recap 

## Sec 6.9: Further Reading 
## Set up factor that identifies the `have' cities 
## Data frame cities (DAAG) 
cities$have <- factor((cities$REGION=="ON")| 
                      (cities$REGION=="WEST")) 
plot(POP1996 ~ POP1992, data=cities,  
     col=as.integer(cities$have)) 
plot(log(POP1996) ~ log(POP1992), data=cities,  
                        col=as.integer(cities$have)) 
cities.lm1 <- lm(POP1996 ~ have+POP1992, data=cities) 
cities.lm2 <- lm(log(POP1996) ~ have+log(POP1992), 
                 data=cities) 
hills.lm <- lm(time ~ dist+climb, data=hills) 
hills2.lm <- lm(time ~ dist+climb+dist:climb,  
                data=hills) 
anova(hills.lm, hills2.lm) 
x1 <- runif(10)            # predictor which will be missing 
x2 <- rbinom(10, 1, 1-x1)  # observed predictor which depends  
                           # on missing predictor  
y <- 5*x1 + x2 + rnorm(10,sd=.1)   # simulated model; coef  
                                   # of x2 is positive  
y.lm <- lm(y ~ factor(x2)) # model fitted to observed data 
coef(y.lm) 
y.lm2 <- lm(y ~ x1 + factor(x2))   # correct model  
coef(y.lm2) 
