## Chapter 9 

## Sec 9.1: Time Series -- Some Basic Ideas 
## ss 9.1.1: Preliminary graphical explorations 
## Plot depth measurements: ts object LakeHuron (datasets) 
plot(LakeHuron, ylab="depth (in feet)", xlab = "Time (in years)") 
lag.plot(LakeHuron, lags=3, do.lines=FALSE) 

## ss 9.1.2: The autocorrelation and partial autocorrelation function 
acf(LakeHuron) 
## pacf(LakeHuron) gives the plot of partial autocorelations 

## ss 9.1.3: Autoregressive (AR) models 
##                       The {\rm AR(1)} model 
## Footnote code 
LH.yw <- ar(x = LakeHuron, order.max = 1, method = "yw") 
                          # autocorrelation estimate 
                          # order.max = 1 for the AR(1) model 
LH.yw$ar                  # autocorrelation estimate of alpha 
LH.mle <- ar(x = LakeHuron, order.max = 1, method = "mle") 
                          # maximum likelihood estimate 
LH.mle$ar                 # maximum likelihood estimate of alpha 
## Estimates of the series mean and of sigma^2 can be obtained by typing 
LH.mle$x.mean             # estimated series mean 
LH.mle$var.pred           # estimated innovation variance 

##                The general {\rm AR(}p{\rm )} model 
ar(LakeHuron, method="mle")  

## ss 9.1.4: {\kern-4pt}$^*$~Autoregressive moving average (ARMA) models -- theory 
## Footnote code 
## Calculate differenced series 
## series.diff <- diff(series) 

## ss 9.1.5: Automatic model selection? 
auto.arima(LakeHuron) 
## Check that model removes most of the correlation structure 
acf(resid(arima(LakeHuron, order=c(1,1,2)))) 
## The following achieves the same effect, for these data 
acf(resid(auto.arima(LakeHuron))) 

## ss 9.1.6: A time series forecast 
LHfit <- auto.arima(LakeHuron) 
fcast <- forecast(LHfit) 
plot(fcast) 

##                    Use of simulation as a check 
oldpar <- par(mfrow=c(3,2), mar=c(3.1,4.6,2.6, 1.1)) 
for(i in 1:4){ 
  ma3 <- 0.125*i 
  simts <- arima.sim(model=list(order=c(0,0,3), ma=c(0,0,ma3)), 
                     n=98) 
  acf(simts, main="", xlab="") 
  mtext(side=3, line=0.5, paste("ma3 =", ma3), adj=0) 
} 
par(oldpar) 
set.seed(29)         # Ensure that results are reproducible 
estMAord <- matrix(0, nrow=4, ncol=20) 
for(i in 1:4){ 
  ma3 <- 0.125*i 
  for(j in 1:20){ 
    simts <- arima.sim(n=98, model=list(ma=c(0,0,ma3))) 
    estMAord[i,j] <- auto.arima(simts, start.q=3)$arma[2] 
} 
} 
detectedAs <- table(row(estMAord), estMAord) 
dimnames(detectedAs) <- list(ma3=paste(0.125*(1:4)),  
                             Order=paste(0:(dim(detectedAs)[2]-1))) 
print(detectedAs) 

## Sec 9.2: {\kern-4pt}$^*$~Regression modeling with arima errors 
## Plot time series mdRain and SOI: ts object bomregions (DAAGxtras) 
plot(ts(bomregions[, c("mdRain","SOI")], start=1900),  
     panel=function(y,...)panel.smooth(bomregions$Year, y,...)) 
xbomsoi <- 
  with(bomregions, data.frame(SOI=SOI, cuberootRain=mdRain^0.33)) 
xbomsoi$trendSOI <- lowess(xbomsoi$SOI, f=0.1)$y 
xbomsoi$trendRain <- lowess(xbomsoi$cuberootRain, f=0.1)$y 
xbomsoi$detrendRain <- 
  with(xbomsoi, cuberootRain - trendRain + mean(trendRain)) 
xbomsoi$detrendSOI <- 
  with(xbomsoi, SOI - trendSOI + mean(trendSOI)) 
## Footnote code 
oldpar <- par(mfrow=c(1,2), pty="s") 
plot(cuberootRain ~ SOI, data = xbomsoi,  
     ylab = "Rainfall (cube root scale)", yaxt="n") 
axis(2, at = rainpos^0.33, labels=paste(rainpos)) 
with(xbomsoi, lines(lowess(cuberootRain ~ SOI, f=0.1))) 
plot(detrendRain ~ detrendSOI, data = xbomsoi, 
  xlab="Detrended SOI", ylab = "Detrended rainfall", yaxt="n") 
rainpos <- pretty(xbomsoi$cuberootRain^3, 6) 
axis(2, at = rainpos^0.33, labels=paste(rainpos))  
with(xbomsoi, lines(lowess(detrendRain ~ detrendSOI, f=0.1))) 
par(oldpar) 
par(mfrow=c(1,2)) 
acf(resid(lm(cuberootRain ~ SOI, data = xbomsoi)), main="") 
pacf(resid(lm(cuberootRain ~ SOI, data = xbomsoi)), main="") 
par(mfrow=c(1,1)) 
auto.arima(residuals(lm(cuberootRain ~ SOI, data=xbomsoi, start.q=5))) 
## Check also residuals from regressing cuberootRain on 
## a quadratic function of SOI 
rain.arima <- with(xbomsoi, Arima(cuberootRain, xreg=SOI, order=c(0,0,1))) 
arima.errors(rain.arima)[1:6] 
auto.arima(arima.errors(rain.arima)) 
xbomsoi.ma1 <- arima(x = cuberootRain, order = c(0, 1, 1), 
                     xreg = SOI) 
xbomsoi.ma1 
## Fit a constant mean model with the same correlation structure 
xbomsoi0.ma1 <- arima(x = cuberootRain, order = c(0, 1, 1)) 
detach(xbomsoi) 
auto.arima(residuals(lm(trendRain ~ trendSOI, data=xbomsoi))) 
with(xbomsoi, arima(trendRain, xreg=trendSOI, order=c(1,1,1))) 

##                   The \texttt{northRain} series 
> northres <- with(bomsoi, resid(lm(northRain^0.33 ~ SOI))) 
> acf(northres) 
> auto.arima(northres) 
Series: northres 
ARIMA(1,1,1)                     
 
Coefficients: 
         ar1      ma1 
      0.1886  -0.9189 
s.e.  0.1092   0.0497 
 
sigma^2 estimated as 0.2098:  log likelihood = -69.05 
AIC = 144.09   AICc = 144.32   BIC = 152.14 

##                   Other checks and computations 
attach(xbomsoi) 
xbomsoi2.ma1 <- arima(x = cuberootRain, order = c(0, 0, 1),  
                      xreg = poly(SOI,2))  
xbomsoi2.ma1 
## Examine normality of estimates of "residuals" ("innovations") 
qqnorm(resid(xbomsoi.ma1, type="normalized")) 
detach(xbomsoi) 

## Sec 9.3: *Nonlinear time series 
x <- numeric(999)  # x will contain the ARCH(1) errors 
x0 <- rnorm(1) 
for (i in 1:999){ 
    x0 <- rnorm(1, sd=sqrt(.25 + .95*x0^2)) 
    x[i] <- x0 
    } 
library(tseries) 
garch(x, order = c(0, 1), trace=FALSE) 

## Sec 9.4: Other time series packages 

## Sec 9.5: Further Reading 

##                         Spatial statistics 
Box.test(resid(lm(detrendRain ~ detrendSOI, data = xbomsoi)), 
         type="Ljung-Box", lag=20) 
xx <- matrix(x, ncol=1000} 
library(tseries) 
data(ice.river)      # Needed with version 0.9-30 of tseries 
river1 <- diff(log(ice.river[, 1])) 
